@@ -2,8 +2,6 @@
 from functools import wraps
 from typing import Callable, List, Union, Any, TypeVar, Optional
 from types import TracebackType
-
-import dill
 import logging
 from tblib import Traceback
 
@@ -113,8 +111,8 @@ class BadStdStreamFile(ParslError):
 class RemoteExceptionWrapper:
     def __init__(self, e_type: type, e_value: BaseException, traceback: Optional[TracebackType]) -> None:
 
-        self.e_type = dill.dumps(e_type)
-        self.e_value = dill.dumps(e_value)
+        self.e_type = e_type
+        self.e_value = e_value
         self.e_traceback = None if traceback is None else Traceback(traceback)
         if e_value.__cause__ is None:
             self.cause = None
@@ -124,20 +122,20 @@ class RemoteExceptionWrapper:
 
     def reraise(self) -> None:
 
-        t = dill.loads(self.e_type)
+        t = self.e_type
 
         # the type is logged here before deserialising v and tb
         # because occasionally there are problems deserialising the
         # value (see #785, #548) and the fix is related to the
         # specific exception type.
-        logger.debug("Reraising exception of type {}".format(t))
+        logger.debug("Reraising exception of type {}".format(self.e_type))
 
         v = self.get_exception()
 
         reraise(t, v, v.__traceback__)
 
-    def get_exception(self) -> Exception:
-        v = dill.loads(self.e_value)
+    def get_exception(self) -> BaseException:
+        v = self.e_value
         if self.cause is not None:
             v.__cause__ = self.cause.get_exception()
         if self.e_traceback is not None:
