@@ -1,10 +1,8 @@
 """This module implements DataFutures.
 """
-import os
 import logging
 from concurrent.futures import Future
 
-from parsl.dataflow.futures import _STATE_TO_DESCRIPTION_MAP, FINISHED
 from parsl.app.errors import NotFutureError
 from parsl.data_provider.files import File
 
@@ -97,63 +95,12 @@ class DataFuture(Future):
         else:
             return False
 
-    def __repr__(self):
-
-        parent = self.parent
-
-        if parent:
-            with parent._condition:
-                if parent._state == FINISHED:
-                    if parent._exception:
-                        return '<%s at %#x state=%s raised %s>' % (
-                            self.__class__.__name__,
-                            id(self),
-                            _STATE_TO_DESCRIPTION_MAP[parent._state],
-                            parent._exception.__class__.__name__)
-                    else:
-                        return '<%s at %#x state=%s with file %s>' % (
-                            self.__class__.__name__,
-                            id(self),
-                            _STATE_TO_DESCRIPTION_MAP[parent._state],
-                            repr(self.file_obj))
-                return '<%s at %#x state=%s>' % (
-                    self.__class__.__name__,
-                    id(self),
-                    _STATE_TO_DESCRIPTION_MAP[parent._state])
-
+    def __repr__(self) -> str:
+        type_ = type(self)
+        module = type_.__module__
+        qualname = type_.__qualname__
+        if self.done():
+            done = "done"
         else:
-            return '<%s at %#x state=%s>' % (
-                self.__class__.__name__,
-                id(self),
-                _STATE_TO_DESCRIPTION_MAP[self._state])
-
-
-def testing_nonfuture():
-    fpath = '~/shuffled.txt'
-    df = DataFuture(None, fpath)
-    print(df)
-    print("Result: ", df.filepath)
-    assert df.filepath == os.path.abspath(os.path.expanduser(fpath))
-
-
-if __name__ == "__main__":
-    # logging.basicConfig(filename='futures.testing.log',level=logging.DEBUG)
-    import sys
-    import random
-    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
-    logger.debug("Begin Testing")
-
-    with open('shuffled.txt', 'w') as testfile:
-        nums = list(range(0, 10000))
-        random.shuffle(nums)
-        for item in nums:
-            testfile.write("{0}\n".format(item))
-
-    foo = Future()  # type: Future[str]
-    df = DataFuture(foo, './shuffled.txt')
-    dx = DataFuture(foo, '~/shuffled.txt')
-
-    print(foo.done())
-    print(df.done())
-
-    testing_nonfuture()
+            done = "not done"
+        return f"<{module}.{qualname} object at {hex(id(self))} representing {repr(self.file_obj)} {done}>"
