@@ -84,7 +84,7 @@ class DataFlowKernel(object):
         if config.initialize_logging:
             parsl.set_file_logger("{}/parsl.log".format(self.run_dir), level=logging.DEBUG)
 
-        logger.debug("Starting DataFlowKernel with config\n{}".format(config))
+        logger.info("Starting DataFlowKernel with config\n{}".format(config))
 
         logger.info("Parsl version: {}".format(get_version()))
 
