@@ -59,20 +59,6 @@ class GoogleCloudProvider():
         scaling where as many resources as possible are used; parallelism close to 0 represents
         the opposite situation in which as few resources as possible (i.e., min_blocks) are used.
 
-    .. code:: python
-
-                                +------------------
-                                |
-          script_string ------->|  submit
-               id      <--------|---+
-                                |
-          [ ids ]       ------->|  status
-          [statuses]   <--------|----+
-                                |
-          [ ids ]       ------->|  cancel
-          [cancel]     <--------|----+
-                                |
-                                +-------------------
      """
 
     def __init__(self,
@@ -105,7 +91,6 @@ class GoogleCloudProvider():
 
         # Dictionary that keeps track of jobs, keyed on job_id
         self.resources = {}
-        self.provisioned_blocks = 0
         atexit.register(self.bye)
 
     def submit(self, command, tasks_per_node, job_name="parsl.gcs"):
@@ -130,7 +115,6 @@ class GoogleCloudProvider():
                                     1)
 
         instance, name = self.create_instance(command=wrapped_cmd)
-        self.provisioned_blocks += 1
         self.resources[name] = {"job_id": name, "status": JobStatus(translate_table[instance['status']])}
         return name
 
@@ -173,16 +157,10 @@ class GoogleCloudProvider():
             try:
                 self.delete_instance(job_id)
                 statuses.append(True)
-                self.provisioned_blocks -= 1
             except Exception:
                 statuses.append(False)
         return statuses
 
-    @property
-    def current_capacity(self):
-        """Returns the number of currently provisioned blocks."""
-        return self.provisioned_blocks
-
     def bye(self):
         self.cancel([i for i in list(self.resources)])
 
