@@ -15,7 +15,6 @@ from parsl.providers import AWSProvider
 
 from parsl.config import Config
 from parsl.executors import HighThroughputExecutor
-from parsl.tests.utils import get_rundir
 
 # If you are a developer running tests, make sure to update parsl/tests/configs/user_opts.py
 # If you are a user copying-and-pasting this as an example, make sure to either
@@ -42,6 +41,5 @@ config = Config(
                 walltime='01:00:00',
             ),
         )
-    ],
-    run_dir=get_rundir(),
+    ]
 )
