@@ -13,9 +13,8 @@ import _pytest.runner as runner
 
 import parsl
 from parsl.dataflow.dflow import DataFlowKernelLoader
-from parsl.tests.utils import get_rundir
 
-logger = logging.getLogger('parsl')
+logger = logging.getLogger(__name__)
 
 
 def dumpstacks(sig, frame):
@@ -49,7 +48,6 @@ def pytest_addoption(parser):
         required=True,
         help="run with parsl CONFIG; use 'local' to run locally-defined config"
     )
-    parser.addoption('--bodge-dfk-per-test', action='store_true')
 
 
 def pytest_configure(config):
@@ -105,69 +103,27 @@ def load_dfk_session(request, pytestconfig):
 
     config = pytestconfig.getoption('config')[0]
 
-    if pytestconfig.getoption('bodge_dfk_per_test'):
-        yield
-        return
-
     if config != 'local':
         spec = importlib.util.spec_from_file_location('', config)
-        try:
-            module = importlib.util.module_from_spec(spec)
-            spec.loader.exec_module(module)
-            module.config.run_dir = get_rundir()  # Give unique rundir; needed running with -n=X where X > 1.
+        module = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(module)
 
-            if DataFlowKernelLoader._dfk is not None:
-                raise ValueError("DFK didn't start as None - there was a DFK from somewhere already")
+        if DataFlowKernelLoader._dfk is not None:
+            raise RuntimeError("DFK didn't start as None - there was a DFK from somewhere already")
 
+        if hasattr(module, 'config'):
             dfk = parsl.load(module.config)
+        elif hasattr(module, 'fresh_config'):
+            dfk = parsl.load(module.fresh_config())
+        else:
+            raise RuntimeError("Config module does not define config or fresh_config")
 
-            yield
-
-            if(parsl.dfk() != dfk):
-                raise ValueError("DFK changed unexpectedly during test")
-            dfk.cleanup()
-            parsl.clear()
-        except KeyError:
-            pytest.skip('options in user_opts.py not configured for {}'.format(config))
-    else:
         yield
 
-
-@pytest.fixture(autouse=True, scope='function')
-def load_dfk_bodge_per_test_for_workqueue(request, pytestconfig):
-    """Load a dfk around entire test suite, except in local mode.
-
-    The special path `local` indicates that configuration will not come
-    from a pytest managed configuration file; in that case, see
-    load_dfk_local_module for module-level configuration management.
-    """
-
-    config = pytestconfig.getoption('config')[0]
-
-    if not pytestconfig.getoption('bodge_dfk_per_test'):
-        yield
-        return
-
-    if config != 'local':
-        spec = importlib.util.spec_from_file_location('', config)
-        try:
-            module = importlib.util.module_from_spec(spec)
-            spec.loader.exec_module(module)
-            module.config.run_dir = get_rundir()  # Give unique rundir; needed running with -n=X where X > 1.
-
-            if DataFlowKernelLoader._dfk is not None:
-                raise ValueError("DFK didn't start as None - there was a DFK from somewhere already")
-
-            dfk = parsl.load(module.config)
-
-            yield
-
-            if(parsl.dfk() != dfk):
-                raise ValueError("DFK changed unexpectedly during test")
-            dfk.cleanup()
-            parsl.clear()
-        except KeyError:
-            pytest.skip('options in user_opts.py not configured for {}'.format(config))
+        if(parsl.dfk() != dfk):
+            raise RuntimeError("DFK changed unexpectedly during test")
+        dfk.cleanup()
+        parsl.clear()
     else:
         yield
 
@@ -205,7 +161,7 @@ def load_dfk_local_module(request, pytestconfig):
 
         if(local_config):
             if(parsl.dfk() != dfk):
-                raise ValueError("DFK changed unexpectedly during test")
+                raise RuntimeError("DFK changed unexpectedly during test")
             dfk.cleanup()
             parsl.clear()
 
@@ -281,9 +237,8 @@ def pytest_make_collect_report(collector):
     if not call.excinfo:
         outcome = "passed"
     else:
-        from _pytest import nose
         from _pytest.outcomes import Skipped
-        skip_exceptions = (Skipped,) + nose.get_skip_exceptions()
+        skip_exceptions = (Skipped,)
         if call.excinfo.errisinstance(KeyError):
             outcome = "skipped"
             r = collector._repr_failure_py(call.excinfo, "line").reprcrash
